@@ -1,3 +1,5 @@
+#include <stdlib.h>
+
 
 /******************************************************************
 *              (c) Copyright  1997                                *
@@ -24,7 +26,7 @@
 ****************************************************************/
 
 
-void modulus_maxima(float *extrema, float *wt, int *resoln_ptr,
+void modulus_maxima(double *extrema, double *wt, int *resoln_ptr,
   int *np_ptr )
 {
   int resoln = *resoln_ptr;
@@ -35,7 +37,7 @@ void modulus_maxima(float *extrema, float *wt, int *resoln_ptr,
   int offset;
 
 
-  if(!(abs  = (double *) malloc( np * sizeof(double) )))
+  if(!(abs  = (double *) R_alloc( np , sizeof(double) )))
     error("Memory allocation failed for abs in extrema.c");
 
   for (j = 0; j < resoln; j++)  {
@@ -54,7 +56,6 @@ void modulus_maxima(float *extrema, float *wt, int *resoln_ptr,
 	extrema[offset+x] = 0.0;
     }
   }
-  free( abs );
 }
 
 /****************************************************************
@@ -70,7 +71,7 @@ void modulus_maxima(float *extrema, float *wt, int *resoln_ptr,
 *
 ****************************************************************/
 
-void extrema_input(float *extrema, int max_resoln, int np,
+void extrema_input(double *extrema, int max_resoln, int np,
   image_ext **ext, int *num_of_extrema)
 {
   int j, k, t, length, offset;
@@ -82,7 +83,7 @@ void extrema_input(float *extrema, int max_resoln, int np,
     if ( extrema[t] != 0.0 )
       (*num_of_extrema)++;
 
-  if(!(*ext = (image_ext *) malloc( (*num_of_extrema) * sizeof(image_ext) )))
+  if(!(*ext = (image_ext *) R_alloc( (*num_of_extrema) , sizeof(image_ext) )))
     error("Memory allocation failed for *ext in point_input.c \n");
 
   k = 0; 
