@@ -1,3 +1,5 @@
+#include <stdlib.h>
+
 
 
 /***************************************************************
@@ -19,6 +21,7 @@
 
 
 #include "denoise.h"
+#include "dyadic.h"
 #include "Swave.h"
 
 #define HISTO_SIZE    500
@@ -33,13 +36,13 @@ extern long idum;
 /**************************************************************************/
 #define LOCAL_LENGTH    17
 
-void local_mean(float *mean, float *s, int np )
+void local_mean(double *mean, double *s, int np )
 {
-  float *s_sym;
+  double *s_sym;
   int i, j, t;
-  float sum;
+  double sum;
 
-  if(!(s_sym = (float *) malloc( 2*np*sizeof(float))))
+  if(!(s_sym = (double *) R_alloc( 2*np,sizeof(double))))
     error("Memory allocation failed in s_sym at simul.c \n");
 
   for ( t = 0; t < np; t++ )
@@ -50,10 +53,10 @@ void local_mean(float *mean, float *s, int np )
     sum = 0.0;
     for ( j = t-8, i = 0; i < LOCAL_LENGTH; i++, j++ )
       sum += (j < 0) ? s_sym[-j-1] : s_sym[j];
-    mean[t] = sum / (float) LOCAL_LENGTH;
+    mean[t] = sum / (double) LOCAL_LENGTH;
   }
 
-  free( s_sym );
+  //free( s_sym );
 }
 #undef LOCAL_LENGTH
 
@@ -63,18 +66,18 @@ void local_mean(float *mean, float *s, int np )
 /* variance */
 /**************************************************************************/
 
-float variance(float *s, int np )
+double variance(double *s, int np )
 {
-  float mean, sum;
-  float *temp;
+  double mean, sum;
+  double *temp;
   int i;
 
-  if(!(temp = (float *)malloc(np * sizeof(float))))
+  if(!(temp = (double *)R_alloc(np , sizeof(double))))
     error("Memory allocation failed for temp at simul.c ");
 
   for ( i = 0, mean = 0.0; i < np; i++ )
     mean += s[i];
-  mean /= (float) np;
+  mean /= (double) np;
 
   for ( i = 0; i < np; i++ )
     temp[i] = s[i] - mean;
@@ -82,8 +85,8 @@ float variance(float *s, int np )
   for ( i = 0, sum = 0.0; i < np; i++ )
     sum += temp[i] * temp[i];
 
-  free( temp );
-  return sum / (float) np;
+  //free( temp );
+  return sum / (double) np;
 }
 
 /****************************************************************************/
@@ -91,7 +94,7 @@ float variance(float *s, int np )
 /* Def:  p_value = P{ T > T_obs }                                           */
 /****************************************************************************/
 
-float p_value(float T, float **histo, int resoln, int histo_size )
+double p_value(double T, double **histo, int resoln, int histo_size )
 {
   int count = 0;
   int i;
@@ -102,23 +105,23 @@ float p_value(float T, float **histo, int resoln, int histo_size )
       count = histo_size - i;  /* num of values that > T */
       break;
     }
-  return (float) count / histo_size;
+  return (double) count / histo_size;
 }
 
 /****************************************************************************/
 /* Compute p-value average                                                  */
 /****************************************************************************/
 
-void compute_pval_average(float *pval, float **p, int max_resoln, int np,
+void compute_pval_average(double *pval, double **p, int max_resoln, int np,
   int num_of_windows, int window_size )
 
 {
   int interval_length = window_size / 4;
   int num_of_intervals = np / interval_length;
-  float *temp;
+  double *temp;
   int m, i, j, k;
 
-  if(!(temp = (float *)malloc(num_of_intervals * sizeof(float))))
+  if(!(temp = (double *)R_alloc(num_of_intervals , sizeof(double))))
     error("Memory allocation failed for temp at simul.c \n");
 
   for ( j = 1; j <= max_resoln; j++ )
@@ -145,7 +148,7 @@ void compute_pval_average(float *pval, float **p, int max_resoln, int np,
 	pval[m] = temp[i];
     }
   }
-  free( temp );
+  //free( temp );
 }
 
 
@@ -153,11 +156,11 @@ void compute_pval_average(float *pval, float **p, int max_resoln, int np,
 /****************************************************************************/
 /* Gaussian random variables generator                                      */
 /****************************************************************************/
-float gasdev(long *idum)
+double gasdev(long *idum)
 {
   static int iset=0;
-  static float gset;
-  float fac,rsq,v1,v2;
+  static double gset;
+  double fac,rsq,v1,v2;
 
   if  (iset == 0) {
     do {
@@ -182,9 +185,9 @@ float gasdev(long *idum)
 /* denominator of the statistics = (Wf[1]^2) + (Wf[2]^2)                    */
 /****************************************************************************/
 
-float denominator(float *Wf, int np )
+double denominator(double *Wf, int np )
 {
-  float den = 0.0;
+  double den = 0.0;
   int t;
 
   for ( t = 0; t < 2*np; t++ )  /* resoln 1 and 2 */
@@ -196,33 +199,33 @@ float denominator(float *Wf, int np )
 /* numerator of the statistics = sqrt of (Wf[j]^4)                          */
 /****************************************************************************/
 
-float numerator(float *Wf, int resoln, int np )
+double numerator(double *Wf, int resoln, int np )
 {
-  float sum = 0.0;
+  double sum = 0.0;
   int t, i;
 
   for ( t = (resoln-1)*np, i = 0; i < np; i++, t++ )
     sum += Wf[t] * Wf[t] * Wf[t] * Wf[t];
-  return (float) sqrt( (double) sum );
+  return (double) sqrt( (double) sum );
 }
 
 /***************************************************************************/
 /* Compute mallat normal histogram for computing p-values */
 /***************************************************************************/
 
-void normal_histo( float ***histo, int max_resoln, int sample_size )
+void normal_histo( double ***histo, int max_resoln, int sample_size )
 {
-  float *Sf = (float *) malloc( (max_resoln+1) * sample_size * sizeof(float) );
-  float *Wf = (float *) malloc( max_resoln * sample_size * sizeof(float) );
-  float *sample = (float *) malloc( sample_size * sizeof(float) );
+  double *Sf = (double *) R_alloc( (max_resoln+1) * sample_size , sizeof(double) );
+  double *Wf = (double *) R_alloc( max_resoln * sample_size , sizeof(double) );
+  double *sample = (double *) R_alloc( sample_size , sizeof(double) );
 
-  float den;
+  double den;
   int b, i, j;
 
 
-  *histo = (float **) malloc( (max_resoln+1) * sizeof(float *) );
+  *histo = (double **) R_alloc( (max_resoln+1) , sizeof(double *) );
   for ( j = 1; j <= max_resoln; j++ )
-    (*histo)[j] = (float *) malloc( HISTO_SIZE * sizeof(float) );
+    (*histo)[j] = (double *) R_alloc( HISTO_SIZE , sizeof(double) );
 
   for ( b = 0; b < HISTO_SIZE; b++ )
   {
@@ -240,26 +243,28 @@ void normal_histo( float ***histo, int max_resoln, int sample_size )
   for ( j = 1; j <= max_resoln; j++ )
     qcksrt( HISTO_SIZE, (*histo)[j]-1 );
 
+/*
   free( Sf );
   free( Wf );
   free( sample );
+*/
 }
 
 /***************************************************************************/
 /* Compute mallat bootstrap histogram for computing p-values */
 /***************************************************************************/
 
-void bootstrap_histo(float ***histo, float *s, int max_resoln,
+void bootstrap_histo(double ***histo, double *s, int max_resoln,
   int sample_size )
 {
-  float *Sf = (float *) malloc( (max_resoln+1) * sample_size * sizeof(float) );
-  float *Wf = (float *) malloc( max_resoln * sample_size * sizeof(float) );
+  double *Sf = (double *) R_alloc( (max_resoln+1) * sample_size , sizeof(double) );
+  double *Wf = (double *) R_alloc( max_resoln * sample_size , sizeof(double) );
 
-  float *sample = (float *) malloc( sample_size * sizeof(float) );
-  float *bsample = (float *) malloc( sample_size * sizeof(float) );
-  float *mean = (float *) malloc( sample_size * sizeof(float) );
+  double *sample = (double *) R_alloc( sample_size , sizeof(double) );
+  double *bsample = (double *) R_alloc( sample_size , sizeof(double) );
+  double *mean = (double *) R_alloc( sample_size , sizeof(double) );
 
-  float den;
+  double den;
   int b, i, j;
   int k = sample_size - 16;  
 
@@ -269,9 +274,9 @@ void bootstrap_histo(float ***histo, float *s, int max_resoln,
   for ( i = 0; i < sample_size; i++ )
     bsample[i] -= mean[i];
 
-  *histo = (float **) malloc( (max_resoln+1) * sizeof(float *) );
+  *histo = (double **) R_alloc( (max_resoln+1) , sizeof(double *) );
   for ( j = 1; j <= max_resoln; j++ )
-    (*histo)[j] = (float *) malloc( HISTO_SIZE * sizeof(float) );
+    (*histo)[j] = (double *) R_alloc( HISTO_SIZE , sizeof(double) );
 
   for ( b = 0; b < HISTO_SIZE; b++ )
   {
@@ -289,55 +294,59 @@ void bootstrap_histo(float ***histo, float *s, int max_resoln,
   for ( j = 1; j <= max_resoln; j++ )
     qcksrt( HISTO_SIZE, (*histo)[j]-1 );
 
+/*
   free( Sf );
   free( Wf );
   free( bsample );
   free( mean );
   free( sample );
+*/
 }
 
 /***************************************************************************/
 /* Compute mallat pvalue */
 /***************************************************************************/
 
-void normal_pval_compute(float *pval, float *s, int *max_resoln_ptr,
+void normal_pval_compute(double *pval, double *s, int *max_resoln_ptr,
   int *np_ptr, int *num_of_windows_ptr, int *window_size_ptr )
 {
   int max_resoln = *max_resoln_ptr;
   int np = *np_ptr;
   int num_of_windows = *num_of_windows_ptr;
   int window_size = *window_size_ptr;
-  char **pfiltername = (char **) malloc(sizeof(char *));
+  char *pfiltername = "Gaussian1";
+  //char *pfiltername = (char *) R_alloc(1, sizeof(char *));
 
-  float *window_data;
-  float *Sf;
-  float *Wf;
-  float **p;
+  double *window_data;
+  double *Sf;
+  double *Wf;
+  double **p;
 
   int step = window_size / 4;
 
-  float T, den, **histo;
+  double T, den;
+  double **histo;
   int w, i, j, t;
 
-  if(!(window_data = (float *) malloc( window_size * sizeof(float) )))
+  if(!(window_data = (double *) R_alloc( window_size , sizeof(double) )))
     error("Memory allocation failed for window_data in simul.c \n");
-  if(!(histo = (float **)malloc((max_resoln + 1) * sizeof(float *))))
+  if(!(histo = (double **)R_alloc((max_resoln + 1) , sizeof(double *))))
     error("Memory allocation failed for histo in simul.c \n");
-  if(!(pfiltername = (char **) malloc(sizeof(char *))))
-    error("Memory allocation failed for pfiltername in simul.c \n");
-  if(!(Sf = (float *) malloc((max_resoln+1)* window_size * sizeof(float))))
+  //if(!(pfiltername = (char **) R_alloc(1, sizeof(char *))))
+   // error("Memory allocation failed for pfiltername in simul.c \n");
+  if(!(Sf = (double *) R_alloc((max_resoln+1)* window_size , sizeof(double))))
     error("Memory allocation failed for *Sf in simul.c \n");
-  if(!(Wf = (float *) malloc(max_resoln* window_size * sizeof(float))))
+  if(!(Wf = (double *) R_alloc(max_resoln* window_size , sizeof(double))))
     error("Memory allocation failed for *Wf in simul.c \n");
-  if(!(p = (float **) malloc( (max_resoln+1) * sizeof(float *) )))
+  if(!(p = (double **) R_alloc( (max_resoln+1) , sizeof(double *) )))
     error("Memory allocation failed for p in simul.c \n");
 
 
 
   normal_histo( &histo, max_resoln, window_size );
-  filename_given(*pfiltername,"Gaussian1");
+  //filename_given(*pfiltername,"Gaussian1");
   for ( j = 1; j <= max_resoln; j++ )
-    if(!(p[j] = (float *) malloc( num_of_windows * sizeof(float) )))
+    if(!(p[j] = (double *) R_alloc( num_of_windows , sizeof(double) )))
       error("Memory failed for p[j] in simul.c ");
 
   for ( w = 0; w < num_of_windows; w++ )
@@ -358,6 +367,7 @@ void normal_pval_compute(float *pval, float *s, int *max_resoln_ptr,
 
   compute_pval_average( pval, p, max_resoln, np, num_of_windows, window_size );
 
+/*
   free( window_data );
   free( Sf );
   free( Wf );
@@ -368,13 +378,14 @@ void normal_pval_compute(float *pval, float *s, int *max_resoln_ptr,
   }
   free( histo );
   free( p );
+*/
 }
 
 /***************************************************************************/
 /* Compute mallat bootstrap pvalue */
 /***************************************************************************/
 
-void bootstrap_pval_compute(float *pval, float *s, int *max_resoln_ptr,
+void bootstrap_pval_compute(double *pval, double *s, int *max_resoln_ptr,
   int *np_ptr, int *num_of_windows_ptr, int *window_size_ptr )
 {
   int max_resoln = *max_resoln_ptr;
@@ -382,37 +393,38 @@ void bootstrap_pval_compute(float *pval, float *s, int *max_resoln_ptr,
   int num_of_windows = *num_of_windows_ptr;
   int window_size = *window_size_ptr;
 
-  float *window_data;
-  float *Sf;
-  float *Wf;
-  float **p;
-  char **pfiltername;
+  double *window_data;
+  double *Sf;
+  double *Wf;
+  double **p;
+  char *pfiltername = "Gaussian1";
 
   int step = window_size / 4;
 
-  float T, den, **histo;
+  double T, den;
+  double **histo;
   int w, i, j, offset;
 
-  if(!(window_data = (float *) malloc( window_size * sizeof(float) )))
+  if(!(window_data = (double *) R_alloc( window_size , sizeof(double) )))
     error("Memory allocation failed for window_data in simul.c \n");
-  if(!(histo = (float **)malloc((max_resoln + 1) * sizeof(float *))))
+  if(!(histo = (double **)R_alloc((max_resoln + 1) , sizeof(double *))))
     error("Memory allocation failed for histo in simul.c \n");
-  if(!(pfiltername = (char **) malloc(sizeof(char *))))
+  //if(!(pfiltername = (char **) R_alloc(1, sizeof(char *))))
     error("Memory allocation failed for pfiltername in simul.c \n");
-  if(!(Sf = (float *) malloc((max_resoln+1)* window_size * sizeof(float))))
+  if(!(Sf = (double *) R_alloc((max_resoln+1)* window_size , sizeof(double))))
     error("Memory allocation failed for *Sf in simul.c \n");
-  if(!(Wf = (float *) malloc(max_resoln* window_size * sizeof(float))))
+  if(!(Wf = (double *) R_alloc(max_resoln* window_size , sizeof(double))))
     error("Memory allocation failed for *Wf in simul.c \n");
-  if(!(p = (float **) malloc( (max_resoln+1) * sizeof(float *) )))
+  if(!(p = (double **) R_alloc( (max_resoln+1) , sizeof(double *) )))
     error("Memory allocation failed for p in simul.c \n");
 
   bootstrap_histo( &histo, s, max_resoln, window_size );
 
   for ( j = 1; j <= max_resoln; j++ )
-    if(!(p[j] = (float *) malloc( num_of_windows * sizeof(float) )))
+    if(!(p[j] = (double *) R_alloc( num_of_windows , sizeof(double) )))
       error("Memory allocation failed for p[j] in simul.c \n ");
 
-  filename_given(*pfiltername,"Gaussian1");
+  //filename_given(*pfiltername,"Gaussian1");
 
   for ( w = 0; w < num_of_windows; w++ )
   {
@@ -433,6 +445,7 @@ void bootstrap_pval_compute(float *pval, float *s, int *max_resoln_ptr,
 
   compute_pval_average( pval, p, max_resoln, np, num_of_windows, window_size );
 
+/*
   free( window_data );
   free( Sf );
   free( Wf );
@@ -443,39 +456,40 @@ void bootstrap_pval_compute(float *pval, float *s, int *max_resoln_ptr,
   }
   free( histo );
   free( p );
+*/
 }
 
 /***************************************************************************/
 /* Compute mallat normal threshold for trimming */
 /***************************************************************************/
 
-void nthresh_compute(float *nthresh, float *s, int *maxresoln_ptr,
-  int *sample_size_ptr, float prct )
+void nthresh_compute(double *nthresh, double *s, int *maxresoln_ptr,
+  int *sample_size_ptr, double prct )
 {
   int max_resoln = *maxresoln_ptr;
   int sample_size = *sample_size_ptr;
 
-  float *mean;
-  float *sample;
-  float **histo;
-  float *Sf;
-  float *Wf;
-  char **pfiltername;
+  double *mean;
+  double *sample;
+  double **histo;
+  double *Sf;
+  double *Wf;
+  char *pfiltername = "Gaussian1";
   
-  float var, std;
+  double var, std;
   int j, b, i, t;
 
-  if(!(histo = (float **)malloc((max_resoln + 1) * sizeof(float *))))
+  if(!(histo = (double **)R_alloc((max_resoln + 1) , sizeof(double *))))
     error("Memory allocation failed for histo in simul.c \n");
-  if(!(pfiltername = (char **) malloc(sizeof(char *))))
-    error("Memory allocation failed for pfiltername in simul.c \n");
-  if(!(mean = (float *) malloc( sample_size * sizeof(float))))
+  //if(!(pfiltername = (char **) R_alloc(1, sizeof(char *))))
+    //error("Memory allocation failed for pfiltername in simul.c \n");
+  if(!(mean = (double *) R_alloc( sample_size , sizeof(double))))
     error("Memory allocation failed for *mean in simul.c \n");
-  if(!(sample = (float *) malloc( sample_size * sizeof(float))))
+  if(!(sample = (double *) R_alloc( sample_size , sizeof(double))))
     error("Memory allocation failed for *sample in simul.c \n");
-  if(!(Sf = (float *) malloc((max_resoln+1)* sample_size * sizeof(float))))
+  if(!(Sf = (double *) R_alloc((max_resoln+1)* sample_size , sizeof(double))))
     error("Memory allocation failed for *Sf in simul.c \n");
-  if(!(Wf = (float *) malloc(max_resoln* sample_size * sizeof(float))))
+  if(!(Wf = (double *) R_alloc(max_resoln* sample_size , sizeof(double))))
     error("Memory allocation failed for *Wf in simul.c \n");
 
   /*  printf("Idum = %d\n",idum);  */
@@ -488,17 +502,17 @@ void nthresh_compute(float *nthresh, float *s, int *maxresoln_ptr,
     sample[i] -= mean[i];
 
   var = variance( sample, sample_size );
-  std = (float) sqrt( var );
+  std = (double) sqrt( var );
 
   for ( j = 1; j <= max_resoln; j++ )
-    if(!(histo[j] = (float *) malloc( HISTO_SIZE * sizeof(float) )))
+    if(!(histo[j] = (double *) R_alloc( HISTO_SIZE , sizeof(double) )))
       error("Memory allocation failed for histo[i] in simul.c \n");
 
-  if(!(*pfiltername = (char *)malloc(STRING_SIZE * sizeof(char))))
-    error("Memory allocation failed for *pfilename in simul.c \n");
+  //if(!(*pfiltername = (char *)R_alloc(STRING_SIZE , sizeof(char))))
+   // error("Memory allocation failed for *pfilename in simul.c \n");
 
 
-  filename_given(*pfiltername,"Gaussian1");
+  //filename_given(*pfiltername,"Gaussian1");
 
   for ( b = 0; b < HISTO_SIZE; b++ )  {
     for ( i = 0; i < sample_size; i++ )
@@ -524,49 +538,50 @@ void nthresh_compute(float *nthresh, float *s, int *maxresoln_ptr,
     qcksrt( HISTO_SIZE, histo[j]-1 );
     /*    nthresh[j-1] = histo[j][(int)(HISTO_SIZE * 0.95) - 1];   */
     nthresh[j-1] = histo[j][(int)(HISTO_SIZE * prct) - 1];
-    free( histo[j] );
+    //free( histo[j] );
   }
 
+/*
   free( histo );
   free( mean );
   free( sample );
   free( Sf );
   free( Wf );
+*/
 }
 
 /***************************************************************************/
 /* Compute mallat bootstrap threshold for trimming */
 /***************************************************************************/
 
-void bthresh_compute(float *bthresh, float *s, int *maxresoln_ptr,
-  int *sample_size_ptr, float prct )
+void bthresh_compute(double *bthresh, double *s, int *maxresoln_ptr,
+  int *sample_size_ptr, double prct )
 {
   int max_resoln = *maxresoln_ptr;
   int sample_size = *sample_size_ptr;
 
-  float *mean;
-  float *sample;
-  float *bsample;
-  float **histo;
-  float *Sf;
-  float *Wf;
-  char **pfiltername;
+  double *mean;
+  double *sample;
+  double *bsample;
+  double **histo;
+  double *Sf;
+  double *Wf;
+  char *pfiltername = "Gaussian1";
   int k = sample_size - 16;  /* k depends on LOCAL_LENGTH in local_mean */
   int j, b, i, t;
 
-  if(!(histo = (float **)malloc((max_resoln + 1) * sizeof(float *))))
-    error("Memory allocation failed for histo in simul.c \n");
-  if(!(pfiltername = (char **) malloc(sizeof(char *))))
-    error("Memory allocation failed for pfiltername in simul.c \n");
-  if(!(mean = (float *) malloc( sample_size * sizeof(float))))
+  histo = (double **) R_alloc((max_resoln + 1), sizeof(double *));
+  //if(!(pfiltername = (char **) R_alloc(1, sizeof(char *))))
+    //error("Memory allocation failed for pfiltername in simul.c \n");
+  if(!(mean = (double *) R_alloc( sample_size, sizeof(double))))
     error("Memory allocation failed for *mean in simul.c \n");
-  if(!(sample = (float *) malloc( sample_size * sizeof(float))))
+  if(!(sample = (double *) R_alloc( sample_size , sizeof(double))))
     error("Memory allocation failed for *sample in simul.c \n");
-  if(!(bsample = (float *) malloc( sample_size * sizeof(float))))
+  if(!(bsample = (double *) R_alloc( sample_size , sizeof(double))))
     error("Memory allocation failed for *bample in simul.c \n");
-  if(!(Sf = (float *) malloc((max_resoln+1)* sample_size * sizeof(float))))
+  if(!(Sf = (double *) R_alloc((max_resoln+1)* sample_size , sizeof(double))))
     error("Memory allocation failed for *Sf in simul.c \n");
-  if(!(Wf = (float *) malloc(max_resoln* sample_size * sizeof(float))))
+  if(!(Wf = (double *) R_alloc(max_resoln* sample_size , sizeof(double))))
     error("Memory allocation failed for *Wf in simul.c \n");
 
   for ( i = 0; i < sample_size; i++ )
@@ -576,11 +591,11 @@ void bthresh_compute(float *bthresh, float *s, int *maxresoln_ptr,
     bsample[i] -= mean[i];
 
   for ( j = 1; j <= max_resoln; j++ )
-    if(!(histo[j] = (float *) malloc( HISTO_SIZE * sizeof(float) )))
+    if(!(histo[j] = (double *) R_alloc( HISTO_SIZE , sizeof(double) )))
       error("Memory allocation failed for histo[i] in simul.c \n");
-  *pfiltername = (char *)malloc(STRING_SIZE * sizeof(char));
+  //*pfiltername = (char *)R_alloc(STRING_SIZE , sizeof(char));
 
-  filename_given(*pfiltername,"Gaussian1");
+  //filename_given(*pfiltername,"Gaussian1");
 
   for ( b = 0; b < HISTO_SIZE; b++ )
   {
@@ -606,13 +621,14 @@ void bthresh_compute(float *bthresh, float *s, int *maxresoln_ptr,
     qcksrt( HISTO_SIZE, histo[j]-1 );
     /*    bthresh[j-1] = histo[j][(int)(HISTO_SIZE * 0.95) - 1];   */
     bthresh[j-1] = histo[j][(int)(HISTO_SIZE * prct) - 1];
-    free( histo[j] );
+    //free( histo[j] );
   }
   
-  free( histo );
+  /*free( histo );
   free( mean );
   free( sample );
   free( bsample );
   free( Sf );
   free( Wf );
+  */
 }
